///////////////////////////////////////////
//
// file:  bmorb.h
//
// Copyright 1998 Johnny Be Good! Software
//
//                John R. Ashmun
//                jbg@kagi.com
//
///////////////////////////////////////////

#include <cstdio>
#include <cstring>
#include "Application.h"

/* first define IMPEXP */
/*	export when building the library */
/*	import otherwise */
#ifdef BUILDING_LIB
#define IMPEXP	__declspec(dllexport)
#else
#define IMPEXP	__declspec(dllimport)
#endif

class IMPEXP BMORB_Messenger;
IMPEXP BMORB_Messenger* OpenService( const char* protocol_dir_name,
                                     const char* transaction_name,
                                     int32*      transaction_ID );

/// BMORB_Messenger - extension to BMessenger
/** Client code uses a BMORB_Messenger
    as if it is a BMessenger, but the
    destination team may be on another
    system running the BeOS.  The
    BMORB_Messenger can deliver a
    BMessage to the destination
    system via bmorbd, the BMORB daemon.
*/
class BMORB_Messenger : public BMessenger
{
   private:
      // Don't want anyone constructing these by default,
      // or using the copy constructor, or assignment either.
      BMORB_Messenger( );

      BMORB_Messenger( const BMORB_Messenger& );

      BMORB_Messenger& operator=( const BMORB_Messenger& );


      // The next two aren't implemented (yet?).

      // (May want to let a smart client pick the ORB it talks to.)
      BMORB_Messenger( ulong signature, team_id team = -1 );

      // (May want to let a remote team talk to my team.)
      BMORB_Messenger( const BHandler* target );

   public:
      /// destructor
      ~BMORB_Messenger( );

      BMORB_Messenger( const BMessenger& Imessenger,
                       const char* Server_Node,
                       const char* Server_Signature );

      /// See BMessenger::SendMessage( ) in The Be Book.
      status_t SendMessage( uint32 command, BHandler* reply_to = NULL ) const;

      /// See BMessenger::SendMessage( ) in The Be Book.
      status_t SendMessage( BMessage* a_message, BHandler* reply_to = NULL,
                            bigtime_t timeout = B_INFINITE_TIMEOUT ) const;

      /// See BMessenger::SendMessage( ) in The Be Book.
      status_t SendMessage( uint32 command, BMessage* reply ) const;

      /// See BMessenger::SendMessage( ) in The Be Book.
      status_t SendMessage( BMessage* a_message, BMessage* reply,
                            bigtime_t send_timeout = B_INFINITE_TIMEOUT,
                            bigtime_t reply_timeout = B_INFINITE_TIMEOUT ) const;

   private:
      char* server_node;
      char* server_signature;
};


/// Function to obtain a BMORB_Messenger
/** Client code supplies the names of
    the BMessageProtocol protocol
    directory and the particular
    BMessageProtocols transaction file.
*/
extern BMORB_Messenger* OpenService( const char* protocol_dir_name,
                                     const char* transaction_name,
                                     int32*      transaction_ID );
